"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const enquirer = require("enquirer");
const base_1 = require("../base");
const application_1 = require("../core/create/application");
const container_1 = require("../core/create/container");
const conf_1 = require("../services/conf");
const logger_1 = require("../services/logger");
const track_1 = require("../services/track");
const util_1 = require("../util");
class Create extends base_1.default {
    constructor() {
        super(...arguments);
        this.name = '';
        this.framework = 'vue';
        this.rdc = '';
        this.rdcRepo = '';
    }
    async preInit() {
        await this.config.runHook('checkUpdate', { id: 'create' });
        const { flags, args } = this.parse(Create);
        const { name } = args;
        this.name = name;
        return flags;
    }
    async initialize() {
        await this.ask();
    }
    async preRun() {
        await util_1.default.asyncExec(`mkdir ${this.name}`);
        process.chdir(this.name);
    }
    async run() {
        track_1.uploadHubble({ cmd: 'create' });
        const opts = {
            name: this.name,
            type: conf_1.default.rdType,
            framework: this.framework,
            rdc: this.rdc,
            rdcRepo: this.rdcRepo,
        };
        let core = null;
        const { RdTypes } = conf_1.default;
        switch (conf_1.default.rdType) {
            case RdTypes.Application:
                core = new application_1.default(opts);
                break;
            case RdTypes.Container:
                core = new container_1.default(opts);
        }
        await core.start();
    }
    async postRun() {
        logger_1.logger.log('Run successfully');
        logger_1.logger.log('Hello from RDE, explore with:');
        logger_1.logger.log('==============================');
        logger_1.logger.log(`$ cd ${this.name}`);
        logger_1.logger.log('$ git remote add origin <https://repo.git>');
        logger_1.logger.log('$ rde serve');
        logger_1.logger.log('==============================');
    }
    async ask() {
        const { RdTypes, frameworks } = conf_1.default;
        if (!this.name) {
            const { name } = await enquirer.prompt({
                type: 'input',
                name: 'name',
                message: 'What is the name of project?',
                required: true,
            });
            this.name = name;
        }
        const { type, framework } = await enquirer.prompt([
            {
                name: 'type',
                message: 'What kind of project to create?',
                type: 'select',
                choices: Object.keys(RdTypes).map(name => ({ name })),
                initial: RdTypes.Application,
            },
            {
                name: 'framework',
                message: 'What kind of framework to use?',
                type: 'select',
                choices: Object.keys(frameworks).map(name => ({ name })),
                initial: 'vue',
            },
        ]);
        const isApp = type === RdTypes.Application;
        if (isApp) {
            const frameworkConf = conf_1.default.frameworks[framework];
            const { rdcStarter } = frameworkConf;
            const { rdcName } = await enquirer.prompt({
                type: 'input',
                name: 'rdcName',
                message: 'What is the name of container on docker hub?',
                required: true,
                initial: rdcStarter,
            });
            const { version } = await enquirer.prompt({
                type: 'input',
                name: 'version',
                message: 'What is the version of container?',
                required: true,
                initial: 'latest',
            });
            this.rdc = `${rdcName}:${version}`;
        }
        if (type === RdTypes.Container) {
            const { rdcRepo } = await enquirer.prompt({
                type: 'input',
                name: 'rdcRepo',
                message: 'What is the repository name on docker hub?',
                required: true,
            });
            this.rdcRepo = rdcRepo;
            const name = conf_1.default.frameworks[this.framework].rdcStarter;
            this.rdc = `${name}:latest`;
        }
        conf_1.default.rdType = type;
        this.framework = framework;
    }
}
Create.description = 'open RDE world';
Create.examples = [
    '$ rde create',
];
Create.flags = Object.assign({}, base_1.default.flags);
Create.args = [{
        name: 'name',
        required: false,
        description: 'project name to create',
    }];
exports.default = Create;
