"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const extend = require("deep-extend");
const fs = require("fs");
const js_beautify_1 = require("js-beautify");
const path = require("path");
const conf_1 = require("../services/conf");
const logger_1 = require("../services/logger");
const rdehook_1 = require("../services/rdehook");
const render_1 = require("../services/render");
const sync_1 = require("../services/sync");
const watcher_1 = require("../services/watcher");
const util_1 = require("../util");
const { resolve } = path;
const { templateDir, integrateDir, runtimeDir, dockerWorkDirRoot, RdTypes, } = conf_1.default;
class DockerRun {
    constructor({ watch }) {
        this.watch = watch;
    }
    async start() {
        // merge rda.config rdc config
        this.config = await this.mergeRdcConf();
        // write variable to rdc.variable.js
        await this.writeVariable(this.config);
        if (conf_1.default.isIntegrate) {
            // template render to .integrated
            await this.renderDir(templateDir, integrateDir);
            await rdehook_1.default.trigger('preIntegrate');
            // compose rde and watch app
            await this.composeRde();
            await rdehook_1.default.trigger('postIntegrate');
        }
        else {
            // template render to runtime
            await this.renderDir(templateDir, runtimeDir);
        }
        if (conf_1.default.rdType === RdTypes.Application) {
            const dirRoot = conf_1.default.useLocal ? conf_1.default.cwd : dockerWorkDirRoot;
            await sync_1.default.mergePkgJson(resolve(dirRoot, 'app', 'package.json'), resolve(dirRoot, conf_1.default.rdeDir, 'package.json'), resolve(dirRoot, conf_1.default.rdeDir));
        }
        if (this.watch && conf_1.default.rdType === RdTypes.Container) {
            await this.watchTemplate(conf_1.default.rdeDir);
        }
    }
    async mergeRdcConf() {
        const { cwd, appConfName, rdcConfName, localCacheDir, } = conf_1.default;
        let config = (conf_1.default.useLocal && conf_1.default.isApp) ? require(resolve(localCacheDir, rdcConfName))
            : require(resolve(cwd, rdcConfName));
        const rdaConfPath = resolve(cwd, appConfName);
        if (fs.existsSync(rdaConfPath)) {
            config = extend(config, require(rdaConfPath));
        }
        conf_1.default.rdMode = config.mode || conf_1.default.RdModes.Integrate;
        return config;
    }
    async writeVariable(config) {
        let variables = {};
        if (conf_1.default.rdType === RdTypes.Application) {
            variables = config.container.variables || {};
        }
        else {
            variables = config.variables || {};
        }
        const dirRoot = conf_1.default.useLocal ? conf_1.default.localCacheDir : dockerWorkDirRoot;
        const variablePath = resolve(dirRoot, 'rdc.variables.js');
        await render_1.default.renderTo('module', {
            obj: variables
        }, variablePath, {
            overwrite: true,
        });
        fs.readFile(variablePath, 'utf-8', (err, data) => {
            if (err) {
                throw err;
            }
            fs.writeFile(variablePath, js_beautify_1.js(data, {}), err => {
                if (err) {
                    throw err;
                }
            });
        });
    }
    async renderDir(from, to) {
        // @ts-ignore
        const { render: rdtRender = {}, container, suites } = this.config;
        const { includes = [] } = rdtRender;
        if (container && rdtRender.validate) {
            const result = rdtRender.validate(container.render);
            if (result !== true) {
                throw Error(result);
            }
        }
        if (suites && suites.length) {
            container.render = container.render || {};
            Object.keys(conf_1.default.frameworks).forEach(framework => {
                container.render[`${framework}Suites`] = suites.filter(item => {
                    if (item.framework === framework) {
                        const arr = item.split('/');
                        let alias = item;
                        if (arr.length > 1) {
                            alias = arr[1];
                        }
                        alias = alias.replace(/[-_]/gim, '').toUpperCase();
                        return {
                            name: item,
                            alias,
                        };
                    }
                    return false;
                });
            });
        }
        const dataView = container ? container.render : rdtRender.mock || {};
        const options = {
            overwrite: true,
            dot: true,
        };
        if (conf_1.default.isIntegrate) {
            options.filter = /.*(?<!\.d\.ts)$/;
        }
        await render_1.default.renderDir(from, Object.assign({}, dataView), includes, to, options, rdtRender.tags);
    }
    // app files should not overwrite template file with same path & name
    async copyApp(filePath, destPath, { to, options }) {
        options = options || {};
        logger_1.debug(`【File Changed】${filePath} ${options}`);
        /**
         * 1. mapping.options.overwrite is not true
         * 2. file or dir exist in template dir
         */
        const appPath = resolve(conf_1.default.cwd, 'app');
        const fileAppPath = path.relative(appPath, filePath);
        const fileTemplatePath = resolve(conf_1.default.templateDir, to, fileAppPath);
        if (!options.overwrite && fs.existsSync(fileTemplatePath)) {
            logger_1.logger.error(`${filePath}: RDA should not overwrite RDC file with same name`);
            return;
        }
        await util_1.default.copy(filePath, destPath, { options });
    }
    async composeRde() {
        logger_1.debug(`merged mappings: ${JSON.stringify(this.config.mappings)}`);
        for (let { from, to, options } of this.config.mappings) {
            const appDir = resolve(conf_1.default.cwd, from);
            const destDir = resolve(integrateDir, to);
            if (!options || options.filter) {
                options = options || {};
                options.overwrite = options.overwrite || false;
                options.filter = /.*(?<!\.d\.ts)$/;
            }
            await util_1.default.copy(appDir, destDir, { options });
        }
        if (this.watch) {
            let { mappings } = this.config;
            new watcher_1.default(mappings, this.copyApp.bind(this)).start();
        }
    }
    async watchTemplate(dest) {
        const mappings = [{
                from: templateDir,
                to: dest
            }];
        new watcher_1.default(mappings, this.renderDir.bind(this)).start();
    }
}
exports.default = DockerRun;
