"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const deepExtend = require("deep-extend");
const fs = require("fs");
const fs_extra_1 = require("fs-extra");
const path = require("path");
const writePkg = require("write-pkg");
const conf_1 = require("../services/conf");
const docker_1 = require("../services/docker");
const logger_1 = require("../services/logger");
const util_1 = require("../util");
const ide_1 = require("./ide");
const message_1 = require("./message");
const render_1 = require("./render");
const { resolve, join } = path;
const { RdTypes, rdcConfName, dockerWorkDirRoot, } = conf_1.default;
class Sync {
    get appConf() {
        return require(resolve(conf_1.default.cwd, conf_1.default.appConfName));
    }
    get from() {
        if (conf_1.default.isApp) {
            const { container } = conf_1.default.getAppConf();
            return container.name;
        }
        if (conf_1.default.rdType === RdTypes.Container) {
            const rdcConfPath = resolve(conf_1.default.cwd, rdcConfName);
            const rdcConf = require(rdcConfPath);
            return rdcConf.nodeVersion || 'node:latest';
        }
    }
    get ports() {
        if (conf_1.default.isApp) {
            const { docker } = conf_1.default.getAppConf();
            return docker.ports || [];
        }
        if (conf_1.default.rdType === RdTypes.Container) {
            const rdcConf = require(join(conf_1.default.cwd, conf_1.default.rdcConfName));
            return rdcConf.docker.ports || [];
        }
    }
    /**
     * sync all
     * used for local dev env, resync is needed when container updates
     * 1. generate Dockerfile to local .cache
     * 2. generate docker-compose to local .cache
     *
     * if rdType is app
     * need to gen extra staged files
     */
    async start({ watch, cmd, skipInstall = false }) {
        await util_1.default.asyncSpawn('mkdir', ['-p', conf_1.default.localCacheDir], {});
        await util_1.default.asyncExec('rm -rf app/node_modules');
        await util_1.default.asyncExec(`rm -rf ${conf_1.default.templateDir}/node_modules`);
        if (conf_1.default.isApp) {
            await docker_1.default.pull(this.appConf.container.name);
            await this.genAppStagedFiles();
        }
        else {
            await util_1.default.copy(resolve(conf_1.default.templateDir, 'package.json'), resolve(conf_1.default.localCacheDir, 'package.json'), {});
        }
        await this.genDevPkgJson();
        await docker_1.default.genDockerFile(conf_1.default.dockerWorkDirRoot, this.from, conf_1.default.localCacheDir, conf_1.default.isApp);
        // @TODO: docker-compose merge
        // needed in order to let container developer write multiple services like mongo
        await docker_1.default.genDockerCompose(conf_1.default.dockerWorkDirRoot, cmd, this.ports, watch, `dev-${conf_1.default.tag}`, conf_1.default.localCacheDir, conf_1.default.isApp, cmd === 'build');
        if (!skipInstall) {
            await this.install();
        }
        await ide_1.default.initSettings(conf_1.default.isApp);
        await this.registerHooks();
    }
    /**
     * 1. copy template/package.json to .cache/package.json
     * 2. copy rde/rdc.config.js to .cache/rdc.config.js
     * 3. merge app optional pkg.json & rdc package.json
     */
    async genAppStagedFiles(createRdc = null) {
        const { cwd, rdcConfName, dockerWorkDirRoot, templateDir, localCacheDir, } = conf_1.default;
        // only create application will pass rdc param
        const image = createRdc || this.appConf.container.name;
        await docker_1.default.copy(image, [{
                from: `${dockerWorkDirRoot}/${templateDir}/package.json`,
                to: localCacheDir,
            }, {
                from: `${dockerWorkDirRoot}/${templateDir}`,
                to: cwd
            }, {
                from: `${dockerWorkDirRoot}/${rdcConfName}`,
                to: localCacheDir,
            }]);
        if (!createRdc) {
            await this.mergePkgJson(resolve(cwd, 'app', 'package.json'), resolve(cwd, localCacheDir, 'package.json'), resolve(localCacheDir));
        }
        // 基本没人用，去除功能
        // await this.genDevcontainer(createRdc)
    }
    async genDevcontainer(createRdc) {
        const rdcConfPath = resolve(conf_1.default.localCacheDir, rdcConfName);
        const rdcConf = require(rdcConfPath);
        const map = {
            devcontainer: 'devcontainer.json',
            'docker-compose': 'docker-compose.yml',
            Dockerfile: 'Dockerfile',
        };
        const from = createRdc || this.from;
        rdcConf.docker = rdcConf.docker || {};
        const ports = createRdc ? rdcConf.docker.ports || [] : this.ports;
        const extensions = rdcConf.extensions || [];
        for (let [key, value] of Object.entries(map)) {
            await render_1.default.renderTo(`devcontainer/${key}`, {
                tag: conf_1.default.tag,
                workDir: dockerWorkDirRoot,
                extensions: JSON.stringify(extensions),
                from,
                ports,
            }, resolve(conf_1.default.cwd, '.devcontainer', value), {
                overwrite: true,
            });
        }
    }
    async registerHooks() {
        if (!fs.existsSync('.git')) {
            await util_1.default.asyncExec('git init');
        }
        fs.writeFileSync(join('.git', 'hooks', 'pre-commit'), `#!/bin/sh

rde lint -s
      `, { encoding: 'UTF-8', mode: '755' });
        fs.writeFileSync(join('.git', 'hooks', 'commit-msg'), `#!/bin/sh

commitMsg=$(cat $1)
rde lint -m "$commitMsg"
      `, { encoding: 'UTF-8', mode: '755' });
    }
    async mergePkgJson(appPkgPath, rdcPkgPath, destPath) {
        logger_1.debug('appPkgPath', appPkgPath);
        logger_1.debug('rdcPkgPath', rdcPkgPath);
        let pkgJson = require(rdcPkgPath);
        if (fs.existsSync(appPkgPath) &&
            fs.statSync(appPkgPath).isFile()) {
            const appPkgJson = require(appPkgPath);
            deepExtend(pkgJson, appPkgJson);
        }
        await writePkg(destPath, pkgJson);
        return pkgJson;
    }
    async genDevPkgJson() {
        const dependenciesAttr = [
            'os',
            'engines',
            'engineStrict',
            'dependencies',
            'devDependencies',
            'peerDependencies',
            'bundledDependencies',
            'optionalDependencies'
        ];
        const devPkgJson = {};
        const pkgJson = require(resolve(conf_1.default.localCacheDir, 'package.json'));
        dependenciesAttr.forEach(item => {
            if (pkgJson[item]) {
                devPkgJson[item] = pkgJson[item];
            }
        });
        await fs_extra_1.writeJson(join(conf_1.default.localCacheDir, 'package-cache.json'), devPkgJson, { spaces: '\t' });
    }
    async install() {
        const { cwd, RdTypes, templateDir, localCacheDir, } = conf_1.default;
        if (conf_1.default.isApp) {
            const cachePkgJsonPath = resolve(localCacheDir, 'package.json');
            if (!fs.existsSync(cachePkgJsonPath)) {
                throw Error(message_1.MRDA.NO_PKG_IN_CACHE_DIR);
            }
            const symPkgJsonPath = resolve(cwd, 'package.json');
            if (fs.existsSync(symPkgJsonPath)) {
                fs.unlinkSync(symPkgJsonPath);
            }
            const appNpmRcPath = resolve('app', '.npmrc');
            const rdcNpmRcPath = resolve(templateDir, '.npmrc');
            if (fs.existsSync(rdcNpmRcPath)) {
                await util_1.default.asyncExec(`cp -f ${rdcNpmRcPath} ${cwd}`);
            }
            if (fs.existsSync(appNpmRcPath)) {
                await util_1.default.asyncExec(`cp -f ${appNpmRcPath} ${cwd}`);
            }
            fs.symlinkSync(cachePkgJsonPath, symPkgJsonPath);
            await util_1.default.asyncSpawn('npm', ['i', '--package-lock', 'false'], {
                cwd,
            });
            fs.unlinkSync(symPkgJsonPath);
        }
        if (conf_1.default.rdType === RdTypes.Container) {
            const runtimePkgJsonPath = resolve(templateDir, 'package.json');
            const runtimeNpmRcPath = resolve(templateDir, '.npmrc');
            const symPkgJsonPath = resolve(cwd, 'package.json');
            if (fs.existsSync(symPkgJsonPath)) {
                fs.unlinkSync(symPkgJsonPath);
            }
            if (fs.existsSync(runtimeNpmRcPath)) {
                await util_1.default.asyncExec(`cp -f ${runtimeNpmRcPath} ${cwd}`);
            }
            fs.symlinkSync(runtimePkgJsonPath, symPkgJsonPath);
            await util_1.default.asyncSpawn('npm', ['i', '--package-lock', 'false'], {
                cwd,
            });
            fs.unlinkSync(symPkgJsonPath);
        }
        if (fs.existsSync(resolve(cwd, '.npmrc'))) {
            await util_1.default.asyncExec('rm -rf .npmrc');
        }
    }
}
exports.default = new Sync();
